@@ -543,22 +543,24 @@ static int get_nodename(struct objdb_iface_ver0 *objdb)
 			error = uname(&utsname);
 			if (error) {
 				sprintf(error_reason, "cannot get node name, uname failed");
-				write_cman_pipe("Can't determine local node name");
+				write_cman_pipe("Can't determine local node name, uname failed");
 				error = -1;
 				goto out;
 			}
 
 			if (strlen(utsname.nodename) >= sizeof(nodename)) {
 				sprintf(error_reason, "node name from uname is too long");
-				write_cman_pipe("Can't determine local node name");
+				write_cman_pipe("local node name is too long");
 				error = -1;
 				goto out;
 			}
 
 			strcpy(nodename, utsname.nodename);
 		}
-		if (verify_nodename(objdb, nodename))
+		if (verify_nodename(objdb, nodename)) {
+			write_cman_pipe("Cannot find node name in cluster.conf");
 			return -1;
+		}
 
 	}
 
@@ -622,8 +624,10 @@ static int get_nodename(struct objdb_iface_ver0 *objdb)
 		free(str);
 	}
 
-	if (add_ifaddr(objdb, mcast_name, nodename, portnum, broadcast))
+	if (add_ifaddr(objdb, mcast_name, nodename, portnum, broadcast)) {
+		write_cman_pipe(error_reason);
 		return -1;
+	}
 
 	/* Get all alternative node names */
 	num_nodenames = 1;
@@ -643,8 +647,10 @@ static int get_nodename(struct objdb_iface_ver0 *objdb)
 			mcast = mcast_name;
 		}
 
-		if (add_ifaddr(objdb, mcast, node, portnum, broadcast))
+		if (add_ifaddr(objdb, mcast, node, portnum, broadcast)) {
+			write_cman_pipe(error_reason);
 			return -1;
+		}
 
 		num_nodenames++;
 	}
@@ -1155,11 +1161,86 @@ err:
 	return ret;
 }
 
+
+static hdb_handle_t find_or_create_object(struct objdb_iface_ver0 *objdb, const char *name, hdb_handle_t parent_handle)
+{
+	hdb_handle_t find_handle;
+	hdb_handle_t ret_handle = 0;
+
+	objdb->object_find_create(parent_handle, name, strlen(name), &find_handle);
+        objdb->object_find_next(find_handle, &ret_handle);
+	objdb->object_find_destroy(find_handle);
+
+	if (!ret_handle) {
+		objdb->object_create(parent_handle, &ret_handle, name, strlen(name));
+	}
+
+	return ret_handle;
+}
+
+static const char *groupd_compat="groupd_compat";
+static const char *clvmd_interface="interface";
+static const char *cman_disallowed="disallowed";
+static const char *totem_crypto="crypto_accept";
+
+/*
+ * Flags to set:
+ * - groupd:
+ * - clvmd
+ * - disallowed (on)
+ * - rgmanager
+ */
+static void setup_old_compat(struct objdb_iface_ver0 *objdb, hdb_handle_t cluster_handle)
+{
+	hdb_handle_t groupd_handle;
+	hdb_handle_t clvmd_handle;
+	hdb_handle_t cman_handle;
+	hdb_handle_t totem_handle;
+	char *value;
+
+	/* Set groupd to backwards compatibility mode */
+	groupd_handle = find_or_create_object(objdb, "group", cluster_handle);
+	if (objdb->object_key_get(groupd_handle, groupd_compat, strlen(groupd_compat),
+				  (void *)&value, NULL) ||
+	    !value) {
+		objdb->object_key_create(groupd_handle, groupd_compat, strlen(groupd_compat),
+					 "1", 2);
+	}
+
+	/* Make clvmd use cman */
+	clvmd_handle = find_or_create_object(objdb, "clvmd", cluster_handle);
+	if (objdb->object_key_get(clvmd_handle, clvmd_interface, strlen(clvmd_interface),
+				  (void *)&value, NULL) ||
+	    !value) {
+		objdb->object_key_create(clvmd_handle, clvmd_interface, strlen(clvmd_interface),
+					 "cman", 5);
+	}
+
+	/* Make cman use disallowed mode */
+	cman_handle = find_or_create_object(objdb, "cman", cluster_handle);
+	if (objdb->object_key_get(cman_handle, cman_disallowed, strlen(cman_disallowed),
+				  (void *)&value, NULL) ||
+	    !value) {
+		objdb->object_key_create(cman_handle, cman_disallowed, strlen(cman_disallowed),
+					 "1", 2);
+	}
+
+	/* Make totem use the old communications method */
+	totem_handle = find_or_create_object(objdb, "totem", OBJECT_PARENT_HANDLE);
+	if (objdb->object_key_get(totem_handle, totem_crypto, strlen(totem_crypto),
+				  (void *)&value, NULL) ||
+	    !value) {
+		objdb->object_key_create(totem_handle, totem_crypto, strlen(totem_crypto),
+					 "old", 4);
+	}
+}
+
 static int cmanpre_readconfig(struct objdb_iface_ver0 *objdb, const char **error_string)
 {
 	int ret = 0;
 	hdb_handle_t object_handle;
 	hdb_handle_t find_handle;
+	char *str;
 
 	if (getenv("CMAN_PIPE"))
                 startup_pipe = atoi(getenv("CMAN_PIPE"));
@@ -1188,6 +1269,12 @@ static int cmanpre_readconfig(struct objdb_iface_ver0 *objdb, const char **error
         }
 	objdb->object_find_destroy(find_handle);
 
+	/* Set up STABLE2/RHEL5 compatibility modes */
+	objdb_get_string(objdb, object_handle, "upgrading", &str);
+	if (str && strcasecmp(str, "on")==0) {
+		setup_old_compat(objdb, cluster_parent_handle);
+	}
+
 	/* This will create /libccs/@next_handle.
 	 * next_handle will be atomically incremented by corosync to return ccs_handle down the pipe.
 	 * We create it in cman-preconfig to avoid an "init" race in libccs.
@@ -1226,6 +1313,12 @@ static int cmanpre_readconfig(struct objdb_iface_ver0 *objdb, const char **error
 	}
         *error_string = error_reason;
 
+
+	/* Close stderr, because cman_tool tells corosync not to.
+	   This helps pass error messages back to the command-line
+	*/
+	if (!debug)
+	    close(STDERR_FILENO);
 	return ret;
 }
 
