@@ -26,6 +26,8 @@
 #define ST_POWEROFF 2
 #define ST_GENERIC_RESET 3
 
+#define DEFAULT_TIMEOUT 10
+
 #define log(lvl, fmt, args...) fprintf(stderr, fmt, ##args)
 #include <libgen.h>
 #include "copyright.cf"
@@ -47,7 +49,7 @@ do { \
 
 
 struct ipmi {
-	char *i_id;
+	const char *i_id;
 	const char *i_ipmitool;
 	char *i_host;
 	char *i_user;
@@ -59,6 +61,8 @@ struct ipmi {
 	int i_config;
 	int i_verbose;
 	int i_lanplus;
+	int i_timeout;
+	int i_cipher;
 };
 
 
@@ -77,10 +81,14 @@ const char *ipmitool_paths[] = {
 };
 
 
+#define ECIPHER 2048
+
 static struct Etoken power_on_complete[] = {
 	{"Password:", EPERM, 0},
 	{"Unable to establish LAN", EAGAIN, 0},	/* Retry */
 	{"IPMI mutex", EFAULT, 0},	/* Death */
+	{"Unsupported cipher suite ID", ECIPHER,0},
+	{"read_rakp2_message: no support for", ECIPHER,0},
 	{"Up/On", 0, 0},
 	{NULL, 0, 0}
 };
@@ -89,6 +97,8 @@ static struct Etoken power_off_complete[] = {
 	{"Password:", EPERM, 0},
 	{"Unable to establish LAN", EAGAIN, 0},	/* Retry */
 	{"IPMI mutex", EFAULT, 0},	/* Death */
+	{"Unsupported cipher suite ID", ECIPHER,0},
+	{"read_rakp2_message: no support for", ECIPHER,0},
 	{"Down/Off", 0, 0},
 	{NULL, 0, 0}
 };
@@ -100,11 +110,31 @@ static struct Etoken power_status[] = {
 	{"Password:", EPERM, 0},
 	{"Unable to establish LAN", EAGAIN, 0},	/* Retry */
 	{"IPMI mutex", EFAULT, 0},	/* Death */
+	{"Unsupported cipher suite ID", ECIPHER,0},
+	{"read_rakp2_message: no support for", ECIPHER,0},
 	{"Chassis Power is off", STATE_OFF, 0},
 	{"Chassis Power is on", STATE_ON, 0},
 	{NULL, 0, 0}
 };
 
+/* Structure describing one xml metadata value*/
+struct xml_parameter_s {
+  const char *name;
+  const char *description;
+};
+
+/* Array of xml metadatas*/
+struct xml_parameter_s xml_parameters[]={
+  {"auth","IPMI Lan Auth type (md5, password, or none)"},
+  {"ipaddr","IPMI Lan IP to talk to"},
+  {"passwd","Password (if required) to control power on IPMI device"},
+  {"passwd_script","Script to retrieve password (if required)"},
+  {"lanplus","Use Lanplus"},
+  {"login","Username/Login (if required) to control power on IPMI device"},
+  {"action","Operation to perform. Valid operations: on, off, reboot, status, list, monitor or metadata"},
+  {"timeout","Timeout (sec) for IPMI operation"},
+  {"cipher","Ciphersuite to use (same as ipmitool -C parameter)"},
+  {"verbose","Verbose mode"}};
 
 /*
    Search for ipmitool
@@ -135,34 +165,78 @@ ipmitool_path(void)
 }
 
 
+/** Prepare string for use in sh style environment. This function take source
+  string and prepend/append quote (') to start/end of source string to dest
+  string. Any occurence of quote in source string is replaced by '\'' sequence.
+  Dest string must be preallocated.
+
+  @param dest Destination string
+  @param source Source string
+  @param max_len Maximum length of data written to dest string (including end 0)
+  @return Pointer to start of destination string.
+*/
+static char *str_prepare_for_sh(char *dest,char *source,int max_len) {
+  char *dest_p=dest;
+  char *max_dest=dest+max_len;
+
+  if (dest_p+1>=max_dest) {*dest_p=0;return dest;}
+  *dest_p++='\'';
+
+  while (*source) {
+    if (*source=='\'') {
+      if (dest_p+4>=max_dest) {*dest_p=0;return dest;}
+
+      memcpy(dest_p,"'\\''",4);dest_p+=4;
+    } else {
+      if (dest_p+1>=max_dest) {*dest_p=0;return dest;}
+
+      *dest_p++=*source;
+    }
+    source++;
+  }
+
+  if (dest_p+2>=max_dest) {*dest_p=0;return dest;}
+
+  *dest_p++='\'';*dest_p=0;
+
+  return dest;
+}
+
 static int
 build_cmd(char *command, size_t cmdlen, struct ipmi *ipmi, int op)
 {
 	char cmd[2048];
 	char arg[2048];
+	char tmp[2048];
 	int x;
 
 	/* Store path */
 	if (ipmi->i_lanplus) {
-		snprintf(cmd, sizeof(cmd), "%s -I lanplus -H %s", 
-				ipmi->i_ipmitool, ipmi->i_host);
+		snprintf(cmd, sizeof(cmd), "%s -I lanplus -H %s",
+				ipmi->i_ipmitool,
+				str_prepare_for_sh(tmp,ipmi->i_host,sizeof(tmp)));
 	} else {
 		snprintf(cmd, sizeof(cmd), "%s -I lan -H %s", ipmi->i_ipmitool,
-				ipmi->i_host);
+				str_prepare_for_sh(tmp,ipmi->i_host,sizeof(tmp)));
 	}
 
 	if (ipmi->i_user) {
-		snprintf(arg, sizeof(arg), " -U %s", ipmi->i_user);
+		snprintf(arg, sizeof(arg), " -U %s", str_prepare_for_sh(tmp,ipmi->i_user,sizeof(tmp)));
 		strncat(cmd, arg, sizeof(cmd) - strlen(arg));
 	}
 
 	if (ipmi->i_authtype) {
-		snprintf(arg, sizeof(arg), " -A %s", ipmi->i_authtype);
+		snprintf(arg, sizeof(arg), " -A %s", str_prepare_for_sh(tmp,ipmi->i_authtype,sizeof(tmp)));
+		strncat(cmd, arg, sizeof(cmd) - strlen(arg));
+	}
+
+	if (ipmi->i_cipher>=0) {
+		snprintf(arg, sizeof(arg), " -C %d", ipmi->i_cipher);
 		strncat(cmd, arg, sizeof(cmd) - strlen(arg));
 	}
 
 	if (ipmi->i_password) {
-		snprintf(arg, sizeof(arg), " -P %s", ipmi->i_password);
+		snprintf(arg, sizeof(arg), " -P %s", str_prepare_for_sh(tmp,ipmi->i_password,sizeof(tmp)));
 		strncat(cmd, arg, sizeof(cmd) - strlen(arg));
 	} else {
 		snprintf(arg, sizeof(arg), " -P ''");
@@ -276,42 +350,31 @@ static int
 ipmi_op(struct ipmi *ipmi, int op, struct Etoken *toklist)
 {
 	char cmd[2048];
-	int retries = 5; 
 	int ret;
 
 	build_cmd(cmd, sizeof(cmd), ipmi, op);
 
 	if (ipmi_spawn(ipmi, cmd) != 0)
 		return -1;
-	ret = ipmi_expect(ipmi, toklist, 120);
+	ret = ipmi_expect(ipmi, toklist, ipmi->i_timeout);
 	ipmi_reap(ipmi);
 
-	while ((ret == EAGAIN || ret == ETIMEDOUT) && retries > 0) {
-		dbg_printf(ipmi, 3, "Sleeping 5 ...\n");
-		sleep(5);
-		dbg_printf(ipmi, 1, "Retrying ...\n");
-		--retries;
-		
-		if (ipmi_spawn(ipmi, cmd) != 0)
-			return -1;
-		ret = ipmi_expect(ipmi, toklist, 120);
-		if (ret == EFAULT) {
-			/* Doomed. */
-			break;
-		}
-		ipmi_reap(ipmi);
-	}
-
 	if (ret == EFAULT) {
 		log(LOG_CRIT, "ipmilan: ipmitool failed to create "
 		    "mutex; unable to complete operation\n");
 		return ret;
 	}
 
-	if (ret == EAGAIN) {
+	if (ret == ECIPHER) {
+		log(LOG_CRIT, "ipmilan: ipmitool failed to operate "
+		    "with ciphersuite %d; unable to complete operation\n",ipmi->i_cipher);
+		return ret;
+	}
+
+	if (ret == ETIMEDOUT) {
 		/*!!! Still couldn't get through?! */
 		log(LOG_WARNING,
-		    "ipmilan: Failed to connect after 30 seconds\n");
+		    "ipmilan: Failed to connect after %d seconds\n",ipmi->i_timeout);
 	}
 
 	return ret;
@@ -321,7 +384,7 @@ ipmi_op(struct ipmi *ipmi, int op, struct Etoken *toklist)
 static int
 ipmi_off(struct ipmi *ipmi)
 {
-	int ret, retries = 5;
+	int ret, retries = 7;
 
 	ret = ipmi_op(ipmi, ST_STATUS, power_status);
 	switch(ret) {
@@ -333,12 +396,12 @@ ipmi_off(struct ipmi *ipmi)
 		return ret;
 	}
 
-	ret = ipmi_op(ipmi, ST_POWEROFF, power_off_complete);
-	if (ret != 0)
-		return ret;
-
 	while (retries>=0) {
-		sleep(5);
+		ret = ipmi_op(ipmi, ST_POWEROFF, power_off_complete);
+		if (ret != 0)
+			return ret;
+
+		sleep(2);
 		--retries;
 		ret = ipmi_op(ipmi, ST_STATUS, power_status);
 
@@ -363,7 +426,7 @@ ipmi_off(struct ipmi *ipmi)
 static int
 ipmi_on(struct ipmi *ipmi)
 {
-	int ret, retries = 5; 
+	int ret, retries = 7;
 
 	ret = ipmi_op(ipmi, ST_STATUS, power_status);
 	switch(ret) {
@@ -375,12 +438,12 @@ ipmi_on(struct ipmi *ipmi)
 		return ret;
 	}
 
-	ret = ipmi_op(ipmi, ST_POWERON, power_on_complete);
-	if (ret != 0)
-		return ret;
-
 	while (retries>=0) {
-		sleep(5);
+		ret = ipmi_op(ipmi, ST_POWERON, power_on_complete);
+		if (ret != 0)
+			return ret;
+
+		sleep(2);
 		--retries;
 		ret = ipmi_op(ipmi, ST_STATUS, power_status);
 
@@ -432,7 +495,8 @@ ipmi_destroy(struct ipmi *i)
  */
 static struct ipmi *
 ipmi_init(struct ipmi *i, char *host, char *authtype,
-	  char *user, char *password, int lanplus, int verbose)
+	  char *user, char *password, int lanplus, int verbose,int timeout,
+	  int cipher)
 {
 	const char *p;
 
@@ -503,6 +567,8 @@ ipmi_init(struct ipmi *i, char *host, char *authtype,
 	i->i_id = IPMIID;
 	i->i_verbose = verbose;
 	i->i_lanplus = lanplus;
+	i->i_timeout = timeout;
+	i->i_cipher = cipher;
 
 	return i;
 }
@@ -511,7 +577,7 @@ ipmi_init(struct ipmi *i, char *host, char *authtype,
 /**
    Remove leading and trailing whitespace from a line of text.
  */
-int
+static int
 cleanup(char *line, size_t linelen)
 {
 	char *p;
@@ -558,14 +624,15 @@ eol:
 /**
    Parse args from stdin.  Dev + devlen + op + oplen must be valid.
  */
-int
+static int
 get_options_stdin(char *ip, size_t iplen,
 		  char *authtype, size_t atlen,
 		  char *passwd, size_t pwlen,
 		  char *pwd_script, size_t pwd_script_len,
 		  char *user, size_t userlen,
 		  char *op, size_t oplen,
-		  int *lanplus, int *verbose)
+		  int *lanplus, int *verbose,int *timeout,
+	          int *cipher)
 {
 	char in[256];
 	int line = 0;
@@ -627,6 +694,14 @@ get_options_stdin(char *ip, size_t iplen,
 				user[0] = 0;
 		} else if (!strcasecmp(name, "lanplus")) {
 			(*lanplus) = 1;
+		} else if (!strcasecmp(name,"timeout")) {
+			if ((sscanf(val,"%d",timeout)!=1) || *timeout<1) {
+			    *timeout=DEFAULT_TIMEOUT;
+			}
+		} else if (!strcasecmp(name,"cipher")) {
+			if ((sscanf(val,"%d",cipher)!=1) || *cipher<0) {
+			    *cipher=-1;
+			}
 		} else if (!strcasecmp(name, "option") ||
 			   !strcasecmp(name, "operation") ||
 			   !strcasecmp(name, "action")) {
@@ -644,20 +719,20 @@ get_options_stdin(char *ip, size_t iplen,
 /**
    Print a message to stderr and call exit(1).
  */
-void
-fail_exit(char *msg)
+static void
+fail_exit(const char *msg)
 {
 	fprintf(stderr, "failed: %s\n", msg);
 	exit(1);
 }
 
-void
+static void
 usage_exit(char *pname)
 {
 printf("usage: %s <options>\n", pname);
 printf("   -A <authtype>  IPMI Lan Auth type (md5, password, or none)\n");
 printf("   -a <ipaddr>    IPMI Lan IP to talk to\n");
-printf("   -i <ipaddr>    IPMI Lan IP to talk to (deprecated, use -i)\n");
+printf("   -i <ipaddr>    IPMI Lan IP to talk to (deprecated, use -a)\n");
 printf("   -p <password>  Password (if required) to control power on\n"
        "                  IPMI device\n");
 printf("   -P             Use Lanplus\n");
@@ -665,7 +740,9 @@ printf("   -S <path>      Script to retrieve password (if required)\n");
 printf("   -l <login>     Username/Login (if required) to control power\n"
        "                  on IPMI device\n");
 printf("   -o <op>        Operation to perform.\n");
-printf("                  Valid operations: on, off, reboot, status\n");
+printf("                  Valid operations: on, off, reboot, status, list or monitor\n");
+printf("   -t <timeout>   Timeout (sec) for IPMI operation (default %d)\n",DEFAULT_TIMEOUT);
+printf("   -C <cipher>    Ciphersuite to use (same as ipmitool -C parameter)\n");
 printf("   -V             Print version and exit\n");
 printf("   -v             Verbose mode\n\n");
 printf("If no options are specified, the following options will be read\n");
@@ -679,11 +756,33 @@ printf("   login=<login>         Same as -u\n");
 printf("   option=<op>           Same as -o\n");
 printf("   operation=<op>        Same as -o\n");
 printf("   action=<op>           Same as -o\n");
+printf("   timeout=<timeout>     Same as -t\n");
+printf("   cipher=<cipher>       Same as -C\n");
 printf("   verbose               Same as -v\n\n");
 	exit(1);
 }
 
 
+/** Print XML metadata of fence agent*/
+static void print_xml_metadata(char *pname) {
+  int i;
+
+  printf("%s\n","<?xml version=\"1.0\" ?>");
+  printf("%s%s%s\n","<resource-agent name=\"",pname,"\" >");
+  printf("%s\n","<parameters>");
+
+  for (i=0;i<(sizeof(xml_parameters)/sizeof(struct xml_parameter_s));i++) {
+    printf("\t<parameter name=\"%s\" unique=\"1\">\n",xml_parameters[i].name);
+
+    printf("\t\t<shortdesc lang=\"C\">");
+    printf("%s",xml_parameters[i].description);
+    printf("</shortdesc>\n");
+    printf("\t</parameter>\n");
+  }
+  printf("%s\n","</parameters>");
+  printf("%s\n","</resource-agent>");
+}
+
 int
 main(int argc, char **argv)
 {
@@ -699,6 +798,9 @@ main(int argc, char **argv)
 	int verbose=0;
 	char *pname = basename(argv[0]);
 	struct ipmi *i;
+	int timeout=DEFAULT_TIMEOUT;
+	int cipher=-1;
+	int print_final_status=1;
 
 	memset(ip, 0, sizeof(ip));
 	memset(authtype, 0, sizeof(authtype));
@@ -710,7 +812,7 @@ main(int argc, char **argv)
 		/*
 		   Parse command line options if any were specified
 		 */
-		while ((opt = getopt(argc, argv, "A:a:i:l:p:S:Po:vV?hH")) != EOF) {
+		while ((opt = getopt(argc, argv, "A:a:i:l:p:S:Po:vV?hHt:C:")) != EOF) {
 			switch(opt) {
 			case 'A':
 				/* Auth type */
@@ -740,6 +842,18 @@ main(int argc, char **argv)
 				/* Operation */
 				strncpy(op, optarg, sizeof(op));
 				break;
+			case 't':
+				/* Timeout */
+				if ((sscanf(optarg,"%d",&timeout)!=1) || timeout<1) {
+				    fail_exit("Timeout option expects positive number parameter");
+				}
+				break;
+			case 'C':
+				/* Ciphersuite */
+				if ((sscanf(optarg,"%d",&cipher)!=1) || cipher<0) {
+				    fail_exit("Ciphersuite option expects positive number parameter");
+				}
+				break;
 			case 'v':
 				verbose++;
 				break;
@@ -763,7 +877,8 @@ main(int argc, char **argv)
 				      passwd, sizeof(passwd),
 					  pwd_script, sizeof(pwd_script),
 				      user, sizeof(user),
-				      op, sizeof(op), &lanplus, &verbose) != 0)
+				      op, sizeof(op), &lanplus, &verbose,&timeout,
+				      &cipher) != 0)
 			return 1;
 	}
 
@@ -798,9 +913,11 @@ main(int argc, char **argv)
 		snprintf(op,sizeof(op), "reboot");
 
 	if (strcasecmp(op, "off") && strcasecmp(op, "on") &&
-	    strcasecmp(op, "status") && strcasecmp(op, "reboot")) {
+	    strcasecmp(op, "status") && strcasecmp(op, "reboot") &&
+	    strcasecmp(op, "monitor") && strcasecmp(op, "list") &&
+	    strcasecmp(op, "metadata")) {
 		fail_exit("operation must be 'on', 'off', 'status', "
-			  "or 'reboot'");
+			  "'reboot', 'list', 'monitor' or 'metadata'");
 	}
 
 	if (strlen(authtype) &&
@@ -813,7 +930,7 @@ main(int argc, char **argv)
 
 
 	/* Ok, set up the IPMI struct */
-	i = ipmi_init(NULL, ip, authtype, user, passwd, lanplus, verbose);
+	i = ipmi_init(NULL, ip, authtype, user, passwd, lanplus, verbose, timeout, cipher);
 	if (!i)
 		fail_exit("Failed to initialize\n");
 
@@ -837,33 +954,47 @@ main(int argc, char **argv)
 		printf("Powering off machine @ IPMI:%s...", ip);
 		fflush(stdout);
 		ret = ipmi_off(i);
-	} else if (!strcasecmp(op, "status")) {
+	} else if (!strcasecmp(op, "status") || !strcasecmp(op, "monitor")) {
 		printf("Getting status of IPMI:%s...",ip);
 		fflush(stdout);
 		ret = ipmi_op(i, ST_STATUS, power_status);
 		switch(ret) {
 		case STATE_ON:
-			printf("Chassis power = On\n");
+		  if (!strcasecmp(op, "status"))
+			    printf("Chassis power = On\n");
 			ret = 0;
 			break;
 		case STATE_OFF:
-			printf("Chassis power = Off\n");
+		  if (!strcasecmp(op, "status"))
+			    printf("Chassis power = Off\n");
 			ret = 0;
 			break;
 		default:
-			printf("Chassis power = Unknown\n");
+		  if (!strcasecmp(op, "status"))
+			    printf("Chassis power = Unknown\n");
 			ret = 1;
 			break;
 		}
+	} else if (!strcasecmp(op, "list")) {
+	  printf("%s\n","N/A");
+	  ret=0;
+	  print_final_status=0;
+	} else if (!strcasecmp(op, "metadata")) {
+	  print_xml_metadata(pname);
+	  ret=0;
+	  print_final_status=0;
 	}
 
 
 out:
 	ipmi_destroy(i);
 	free(i);
-	if (ret == 0)
-		printf("Done\n");
-	else
-		printf("Failed\n");
+
+	if (print_final_status) {
+	  if (ret == 0)
+	    printf("Done\n");
+	  else
+	    printf("Failed\n");
+	}
 	return ret;
 }
