@@ -29,8 +29,6 @@
 
 #define OPTION_STRING ("bdf:g:hkl:mnsu:V")
 
-char *prog_name;
-
 /**
  * This function is for libgfs2's sake.
  */
@@ -46,11 +44,11 @@ void print_it(const char *label, const char *fmt, const char *fmt2, ...)
 
 /**
  * print_usage - print usage info to the user
- *
+ * @prog_name: The name of this program
  */
 
 static void
-print_usage()
+print_usage(const char *prog_name)
 {
 	printf("Usage:\n");
 	printf("\n");
@@ -153,7 +151,7 @@ decode_arguments(int argc, char *argv[], commandline_t *comline)
 			break;
 
 		case 'h':
-			print_usage();
+			print_usage(argv[0]);
 			exit(EXIT_SUCCESS);
 			break;
 
@@ -244,25 +242,33 @@ print_quota(commandline_t *comline,
 	case GQ_UNITS_KILOBYTE:
 		if (sb->sb_bsize == 512)
 			printf("limit: %-10llu warn: %-10lluvalue: %-10llu\n",
-			       q->qu_limit / 2,
-			       q->qu_warn / 2,
-			       q->qu_value / 2);
+			       (unsigned long long)q->qu_limit / 2,
+			       (unsigned long long)q->qu_warn / 2,
+			       (unsigned long long)q->qu_value / 2);
 		else
 			printf("limit: %-10llu warn: %-10lluvalue: %-10llu\n",
+			       (unsigned long long)
 			       q->qu_limit << (sb->sb_bsize_shift - 10),
+			       (unsigned long long)
 			       q->qu_warn << (sb->sb_bsize_shift - 10),
+			       (unsigned long long)
 			       q->qu_value << (sb->sb_bsize_shift - 10));
 		break;
 
 	case GQ_UNITS_FSBLOCK:
 		printf("limit: %-10llu warn: %-10llu value: %-10llu\n",
-		       q->qu_limit, q->qu_warn, q->qu_value);
+		       (unsigned long long)q->qu_limit,
+		       (unsigned long long)q->qu_warn,
+		       (unsigned long long)q->qu_value);
 		break;
 
 	case GQ_UNITS_BASICBLOCK:
 		printf("limit: %-10llu warn: %-10llu value: %-10llu\n",
+		       (unsigned long long)
 		       q->qu_limit << (sb->sb_bsize_shift - 9),
+		       (unsigned long long)
 		       q->qu_warn << (sb->sb_bsize_shift - 9),
+		       (unsigned long long)
 		       q->qu_value << (sb->sb_bsize_shift - 9));
 		break;
 
@@ -283,14 +289,24 @@ read_superblock(struct gfs2_sb *sb, struct gfs2_sbd *sdp)
 		die("Could not open the block device %s: %s\n",
 			sdp->device_name, strerror(errno));
 	}
-	do_lseek(fd, 0x10 * 4096);
-	do_read(fd, buf, PATH_MAX);
+	if (lseek(fd, 0x10 * 4096, SEEK_SET) != 0x10 * 4096) {
+		fprintf(stderr, "bad seek: %s from %s:%d: "
+			"superblock\n",
+			strerror(errno), __FUNCTION__, __LINE__);
+
+		exit(-1);
+	}
+	if (read(fd, buf, PATH_MAX) != PATH_MAX) {
+		fprintf(stderr, "bad read: %s from %s:%d: superblock\n",
+			strerror(errno), __FUNCTION__, __LINE__);
+		exit(-1);
+	}
 	gfs2_sb_in(sb, buf);
 
 	close(fd);
 }
 
-inline void 
+void
 read_quota_internal(int fd, uint32_t id, int id_type, struct gfs2_quota *q)
 {
 	/* seek to the appropriate offset in the quota file and read the 
@@ -308,11 +324,12 @@ read_quota_internal(int fd, uint32_t id, int id_type, struct gfs2_quota *q)
 		die("failed to read from quota file: %s\n", strerror(errno));
 	if (error != sizeof(struct gfs2_quota))
 		die("Couldn't read %lu bytes from quota file at offset %llu\n",
-		    sizeof(struct gfs2_quota), (unsigned long long)offset);
+		    (unsigned long)sizeof(struct gfs2_quota),
+		    (unsigned long long)offset);
 	gfs2_quota_in(q, buf);
 }
 
-inline void 
+static inline void
 write_quota_internal(int fd, uint32_t id, int id_type, struct gfs2_quota *q)
 {
 	/* seek to the appropriate offset in the quota file and read the
@@ -487,12 +504,21 @@ do_reset(struct gfs2_sbd *sdp, commandline_t *comline)
 		return;
 
 	strcpy(sdp->path_name, comline->filesystem);
-	check_for_gfs2(sdp);
+	if (check_for_gfs2(sdp)) {
+		if (errno == EINVAL)
+			fprintf(stderr, "Not a valid GFS2 mount point: %s\n",
+					sdp->path_name);
+		else
+			fprintf(stderr, "%s\n", strerror(errno));
+		exit(-1);
+	}
 	read_superblock(&sdp->sd_sb, sdp);
-	if (!find_gfs2_meta(sdp))
-		mount_gfs2_meta(sdp);
-	lock_for_admin(sdp);
-	
+	if (mount_gfs2_meta(sdp)) {
+		fprintf(stderr, "Error mounting GFS2 metafs: %s\n",
+			strerror(errno));
+		exit(-1);
+	}
+
 	strcpy(quota_file, sdp->metafs_path);
 	strcat(quota_file, "/quota");
 
@@ -546,12 +572,21 @@ do_list(struct gfs2_sbd *sdp, commandline_t *comline)
 		die("need a filesystem to work on\n");
 
 	strcpy(sdp->path_name, comline->filesystem);
-	check_for_gfs2(sdp);
+	if (check_for_gfs2(sdp)) {
+		if (errno == EINVAL)
+			fprintf(stderr, "Not a valid GFS2 mount point: %s\n",
+					sdp->path_name);
+		else
+			fprintf(stderr, "%s\n", strerror(errno));
+		exit(-1);
+	}
 	read_superblock(&sdp->sd_sb, sdp);
-	if (!find_gfs2_meta(sdp))
-		mount_gfs2_meta(sdp);
-	lock_for_admin(sdp);
-	
+	if (mount_gfs2_meta(sdp)) {
+		fprintf(stderr, "Error mounting GFS2 metafs: %s\n",
+			strerror(errno));
+		exit(-1);
+	}
+
 	strcpy(quota_file, sdp->metafs_path);
 	strcat(quota_file, "/quota");
 
@@ -635,12 +670,21 @@ do_get_one(struct gfs2_sbd *sdp, commandline_t *comline, char *filesystem)
 	char quota_file[BUF_SIZE];
 
 	strcpy(sdp->path_name, filesystem);
-	check_for_gfs2(sdp);
+	if (check_for_gfs2(sdp)) {
+		if (errno == EINVAL)
+			fprintf(stderr, "Not a valid GFS2 mount point: %s\n",
+					sdp->path_name);
+		else
+			fprintf(stderr, "%s\n", strerror(errno));
+		exit(-1);
+	}
 	read_superblock(&sdp->sd_sb, sdp);
-	if (!find_gfs2_meta(sdp))
-		mount_gfs2_meta(sdp);
-	lock_for_admin(sdp);
-	
+	if (mount_gfs2_meta(sdp)) {
+		fprintf(stderr, "Error mounting GFS2 metafs: %s\n",
+			strerror(errno));
+		exit(-1);
+	}
+
 	strcpy(quota_file, sdp->metafs_path);
 	strcat(quota_file, "/quota");
 
@@ -737,7 +781,16 @@ do_sync_one(struct gfs2_sbd *sdp, char *filesystem)
 	char *fsname;
 
 	fsname = mp2fsname(filesystem);
-	set_sysfs(fsname, "quota_sync", "1");
+	if (!fsname) {
+		fprintf(stderr, "Couldn't find GFS2 filesystem mounted at %s\n",
+				filesystem);
+		exit(-1);
+	}
+	if (set_sysfs(fsname, "quota_sync", "1")) {
+		fprintf(stderr, "Error writing to sysfs quota sync file: %s\n",
+				strerror(errno));
+		exit(-1);
+	}
 }
 
 /**
@@ -786,7 +839,7 @@ do_set(struct gfs2_sbd *sdp, commandline_t *comline)
 	int fd;
 	uint64_t offset;
 	uint64_t new_value;
-	int error, adj_flag = 0;;
+	int error, adj_flag = 0;
 	char quota_file[BUF_SIZE];
 	char id_str[16];
 	struct stat stat_buf;
@@ -798,12 +851,21 @@ do_set(struct gfs2_sbd *sdp, commandline_t *comline)
 		die("need a new value\n");
 
 	strcpy(sdp->path_name, comline->filesystem);
-	check_for_gfs2(sdp);
+	if (check_for_gfs2(sdp)) {
+		if (errno == EINVAL)
+			fprintf(stderr, "Not a valid GFS2 mount point: %s\n",
+					sdp->path_name);
+		else
+			fprintf(stderr, "%s\n", strerror(errno));
+		exit(-1);
+	}
 	read_superblock(&sdp->sd_sb, sdp);
-	if (!find_gfs2_meta(sdp))
-		mount_gfs2_meta(sdp);
-	lock_for_admin(sdp);
-	
+	if (mount_gfs2_meta(sdp)) {
+		fprintf(stderr, "Error mounting GFS2 metafs: %s\n",
+			strerror(errno));
+		exit(-1);
+	}
+
 	strcpy(quota_file, sdp->metafs_path);
 	strcat(quota_file, "/quota");
 
@@ -933,9 +995,18 @@ do_set(struct gfs2_sbd *sdp, commandline_t *comline)
 	}
 
 	fs = mp2fsname(comline->filesystem);
+	if (!fs) {
+		fprintf(stderr, "Couldn't find GFS2 filesystem mounted at %s\n",
+				comline->filesystem);
+		exit(-1);
+	}
 	sprintf(id_str, "%d", comline->id);
-	set_sysfs(fs, comline->id_type == GQ_ID_USER ?
-		  "quota_refresh_user" : "quota_refresh_group", id_str);
+	if (set_sysfs(fs, comline->id_type == GQ_ID_USER ?
+		  "quota_refresh_user" : "quota_refresh_group", id_str)) {
+		fprintf(stderr, "Error writing to sysfs quota refresh file: %s\n",
+				strerror(errno));
+		exit(-1);
+	}
 	
 	if (adj_flag)
 		adjust_quota_list(fd, comline);
@@ -959,9 +1030,6 @@ main(int argc, char *argv[])
     struct gfs2_sbd sbd, *sdp = &sbd;
 	commandline_t comline;
 
-	prog_name = argv[0];
-	sdp->metafs_mounted = 0;
-
 	memset(sdp, 0, sizeof(struct gfs2_sbd));
 	memset(&comline, 0, sizeof(commandline_t));
 
