@@ -225,9 +225,9 @@ int setup_misc_devices(void);
 /* config.c */
 int setup_ccs(void);
 void close_ccs(void);
-void read_ccs_name(char *path, char *name);
-void read_ccs_yesno(char *path, int *yes, int *no);
-int read_ccs_int(char *path, int *config_val);
+void read_ccs_name(const char *path, char *name);
+void read_ccs_yesno(const char *path, int *yes, int *no);
+int read_ccs_int(const char *path, int *config_val);
 int get_weight(int nodeid, char *lockspace);
 
 /* cpg.c */
@@ -239,7 +239,7 @@ void process_lockspace_changes(void);
 void dlm_send_message(struct lockspace *ls, char *buf, int len);
 int dlm_join_lockspace(struct lockspace *ls);
 int dlm_leave_lockspace(struct lockspace *ls);
-char *msg_name(int type);
+const char *msg_name(int type);
 void update_flow_control_status(void);
 int set_node_info(struct lockspace *ls, int nodeid, struct dlmc_node *node);
 int set_lockspace_info(struct lockspace *ls, struct dlmc_lockspace *lockspace);
@@ -257,10 +257,12 @@ void receive_cycle_start(struct lockspace *ls, struct dlm_header *hd, int len);
 void receive_cycle_end(struct lockspace *ls, struct dlm_header *hd, int len);
 void receive_cancel_lock(struct lockspace *ls, struct dlm_header *hd, int len);
 void deadlk_confchg(struct lockspace *ls,
-	struct cpg_address *member_list, int member_list_entries,
-	struct cpg_address *left_list, int left_list_entries,
-	struct cpg_address *joined_list, int joined_list_entries);
-
+		    const struct cpg_address *member_list,
+		    size_t member_list_entries,
+		    const struct cpg_address *left_list,
+		    size_t left_list_entries,
+		    const struct cpg_address *joined_list,
+		    size_t joined_list_entries);
 
 /* main.c */
 int do_read(int fd, void *buf, size_t count);
@@ -272,7 +274,7 @@ void client_ignore(int ci, int fd);
 void client_back(int ci, int fd);
 struct lockspace *find_ls(char *name);
 struct lockspace *find_ls_id(uint32_t id);
-char *dlm_mode_str(int mode);
+const char *dlm_mode_str(int mode);
 void cluster_dead(int ci);
 
 /* member_cman.c */
@@ -292,6 +294,7 @@ void process_netlink(int ci);
 
 /* plock.c */
 int setup_plocks(void);
+void close_plocks(void);
 void process_plocks(int ci);
 int limit_plocks(void);
 void receive_plock(struct lockspace *ls, struct dlm_header *hd, int len);
@@ -340,10 +343,12 @@ int inline void set_group_mode(void) { return 0; }
 #endif
 
 /* logging.c */
-
 void init_logging(void);
-void setup_logging();
+void setup_logging(void);
 void close_logging(void);
 
+/* crc.c */
+uint32_t cpgname_to_crc(const char *data, int len);
+
 #endif
 
