@@ -1,22 +1,3 @@
-/*
-  Copyright Red Hat, Inc. 2002
-  Copyright Mission Critical Linux, 2000
-
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2, or (at your option) any
-  later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; see the file COPYING.  If not, write to the
-  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
-  MA 02139, USA.
-*/
 /** @file
  * daemon_init function, does sanity checks and calls daemon().
  *
@@ -44,19 +25,14 @@
 #include <sys/errno.h>
 #include <libgen.h>
 #include <signal.h>
-
-/*
- * This should ultimately go in a header file.
- */
-void daemon_init(char *prog);
-int check_pid_valid(pid_t pid, char *prog);
-int check_process_running(char *prog, pid_t * pid);
+#include <daemon_init.h>
 
 /*
  * Local prototypes.
  */
 static void update_pidfile(char *prog);
 static int setup_sigmask(void);
+static char pid_filename[PATH_MAX];
 
 
 int
@@ -168,14 +144,13 @@ update_pidfile(char *prog)
 {
 	FILE *fp = NULL;
 	char *cmd;
-	char filename[PATH_MAX];
 
-	memset(filename, 0, PATH_MAX);
+	memset(pid_filename, 0, PATH_MAX);
 
 	cmd = basename(prog);
-	snprintf(filename, sizeof (filename), "/var/run/%s.pid", cmd);
+	snprintf(pid_filename, sizeof (pid_filename), "/var/run/%s.pid", cmd);
 
-	fp = fopen(filename, "w");
+	fp = fopen(pid_filename, "w");
 	if (fp == NULL) {
 		exit(1);
 	}
@@ -242,3 +217,14 @@ daemon_init(char *prog)
 
 	//mlockall(MCL_CURRENT | MCL_FUTURE);
 }
+
+
+void
+daemon_cleanup(void)
+{
+	if (!strlen(pid_filename))
+		return;
+
+	unlink(pid_filename);
+	memset(pid_filename, 0, sizeof(pid_filename));
+}
