@@ -1,3 +1,7 @@
+#include <time.h>
+#include <list.h>
+#include <restart_counter.h>
+#include <reslist.h>
 #include <resgroup.h>
 #include <rg_locks.h>
 #include <gettid.h>
@@ -10,7 +14,7 @@
 #include <stdint.h>
 #include <vf.h>
 #include <members.h>
-#include <time.h>
+#include <groups.h>
 
 
 /**
@@ -31,13 +35,12 @@ static int central_events = 0;
 
 extern int running;
 extern int shutdown_pending;
+extern int need_reconfigure;
 static int _master = 0;
 static struct dlm_lksb _master_lock;
 static int _xid = 0;
 static event_master_t *mi = NULL;
 
-void hard_exit(void);
-int init_resource_groups(int);
 void flag_shutdown(int sig);
 void flag_reconfigure(int sig);
 
@@ -47,8 +50,8 @@ event_table_t *master_event_table = NULL;
 void
 set_transition_throttling(int nsecs)
 {
-	if (nsecs < 0)
-		nsecs = 0;
+	if (nsecs < 1)
+		nsecs = 1;
 	transition_throttling = nsecs;
 }
 
@@ -67,6 +70,23 @@ central_events_enabled(void)
 }
 
 
+static void
+hard_exit(void)
+{
+	rg_lockall(L_SYS);
+	rg_doall(RG_INIT, 1, "Emergency stop of %s");
+	//vf_shutdown();
+	exit(1);
+}
+
+
+void
+flag_reconfigure(int __attribute__ ((unused)) sig)
+{
+	need_reconfigure++;
+}
+
+
 /**
   Called to handle the transition of a cluster member from up->down or
   down->up.  This handles initializing services (in the local node-up case),
@@ -89,20 +109,20 @@ node_event(int local, int nodeID, int nodeStatus,
 
 		/* Local Node Event */
 		if (nodeStatus == 0) {
-			log_printf(LOG_ERR, "Exiting uncleanly\n");
+			logt_print(LOG_ERR, "Exiting uncleanly\n");
 			hard_exit();
 		}
 
 		if (!rg_initialized()) {
-			if (init_resource_groups(0) != 0) {
-				log_printf(LOG_ERR,
+			if (init_resource_groups(0, 0) != 0) {
+				logt_print(LOG_ERR,
 				       "#36: Cannot initialize services\n");
 				hard_exit();
 			}
 		}
 
 		if (shutdown_pending) {
-			log_printf(LOG_NOTICE, "Processing delayed exit signal\n");
+			logt_print(LOG_NOTICE, "Processing delayed exit signal\n");
 			running = 0;
 			return;
 		}
@@ -118,7 +138,7 @@ node_event(int local, int nodeID, int nodeStatus,
 	 * Nothing to do for events from other nodes if we are not ready.
 	 */
 	if (!rg_initialized()) {
-		log_printf(LOG_DEBUG, "Services not initialized.\n");
+		logt_print(LOG_DEBUG, "Services not initialized.\n");
 		return;
 	}
 
@@ -141,7 +161,7 @@ node_has_fencing(int nodeid)
 	
 	ccs_desc = ccs_connect();
 	if (ccs_desc < 0) {
-		log_printf(LOG_ERR, "Unable to connect to ccsd; cannot handle"
+		logt_print(LOG_ERR, "Unable to connect to ccsd; cannot handle"
 		       " node event!\n");
 		/* Assume node has fencing */
 		return 1;
@@ -167,33 +187,13 @@ fence_domain_joined(void)
 {
 	int rv;
 
-	rv = system("group_tool ls fence default &> /dev/null");	
+	rv = system("fence_tool ls &> /dev/null");	
 	if (rv == 0)
 		return 1;
 	return 0;
 }
 
 
-/**
-   Quick query to cman to see if a node has been fenced.
- */
-int
-node_fenced(int nodeid)
-{
-	cman_handle_t ch;
-	int fenced = 0;
-	uint64_t fence_time;
-
-	ch = cman_init(NULL);
-	if (cman_get_fenceinfo(ch, nodeid, &fence_time, &fenced, NULL) < 0)
-		fenced = 0;
-
-	cman_finish(ch);
-
-	return fenced;
-}
-
-
 /**
    Callback from view-formation when a commit occurs for the Transition-
    Master key.
@@ -207,20 +207,20 @@ master_event_callback(char __attribute__ ((unused)) *key,
 
 	m = data;
 	if (datalen != (uint32_t)sizeof(*m)) {
-		log_printf(LOG_ERR, "%s: wrong size\n", __FUNCTION__);
+		logt_print(LOG_ERR, "%s: wrong size\n", __FUNCTION__);
 		return 1;
 	}
 
 	swab_event_master_t(m);
 	if (m->m_magic != EVENT_MASTER_MAGIC) {
-		log_printf(LOG_ERR, "%s: wrong size\n", __FUNCTION__);
+		logt_print(LOG_ERR, "%s: wrong size\n", __FUNCTION__);
 		return 1;
 	}
 
 	if (m->m_nodeid == (uint32_t)my_id())
-		log_printf(LOG_DEBUG, "Master Commit: I am master\n");
+		logt_print(LOG_DEBUG, "Master Commit: I am master\n");
 	else 
-		log_printf(LOG_DEBUG, "Master Commit: %d is master\n", m->m_nodeid);
+		logt_print(LOG_DEBUG, "Master Commit: %d is master\n", m->m_nodeid);
 
 	pthread_mutex_lock(&mi_mutex);
 	if (mi)
@@ -249,7 +249,7 @@ find_master(void)
 	m = member_list();
 	if (vf_read(m, "Transition-Master", &vn,
 		    (void **)(&data), &sz) < 0) {
-		log_printf(LOG_ERR, "Unable to discover master"
+		logt_print(LOG_ERR, "Unable to discover master"
 		       " status\n");
 		masterinfo = NULL;
 	} else {
@@ -260,7 +260,7 @@ find_master(void)
 	if (masterinfo && (sz >= sizeof(*masterinfo))) {
 		swab_event_master_t(masterinfo);
 		if (masterinfo->m_magic == EVENT_MASTER_MAGIC) {
-			log_printf(LOG_DEBUG, "Master Locate: %d is master\n",
+			logt_print(LOG_DEBUG, "Master Locate: %d is master\n",
 			       masterinfo->m_nodeid);
 			pthread_mutex_lock(&mi_mutex);
 			if (mi)
@@ -322,7 +322,7 @@ event_master(void)
 		master_id = mi->m_nodeid;
 		pthread_mutex_unlock(&mi_mutex);
 		if (memb_online(m, master_id)) {
-			//log_printf(LOG_DEBUG, "%d is master\n", mi->m_nodeid);
+			//logt_print(LOG_DEBUG, "%d is master\n", mi->m_nodeid);
 			goto out;
 		}
 	}
@@ -353,7 +353,7 @@ event_master(void)
 	if (vf_write(m, VFF_IGN_CONN_ERRORS | VFF_RETRY,
 		     "Transition-Master", &masterinfo,
 		     sizeof(masterinfo)) < 0) {
-		log_printf(LOG_ERR, "Unable to advertise master"
+		logt_print(LOG_ERR, "Unable to advertise master"
 		       " status to all nodes\n");
 	}
 
@@ -364,34 +364,24 @@ out:
 }
 
 
-
-void group_event(char *name, uint32_t state, int owner);
-
 /**
   Event handling function.  This only stays around as long as
   events are on the queue.
  */
-void *
+static void *
 _event_thread_f(void __attribute__ ((unused)) *arg)
 {
 	event_t *ev;
 	struct timeval now;
 	struct timespec expire;
-	int notice = 0, count = 0;
-
-	/* Event thread usually doesn't hang around.  When it's
-   	   spawned, sleep for this many seconds in order to let
-   	   some events queue up */
-	if (transition_throttling && !central_events) {
-		sleep(transition_throttling);
-	}
+	int count = 0;
 
 	while (1) {
 		pthread_mutex_lock(&event_queue_mutex);
 		ev = event_queue;
 		if (!ev && !central_events) {
 			gettimeofday(&now, NULL);
-			expire.tv_sec = now.tv_sec + 5;
+			expire.tv_sec = now.tv_sec + transition_throttling;
 			expire.tv_nsec = now.tv_usec * 1000;
 			pthread_cond_timedwait(&event_queue_cond,
 						&event_queue_mutex,
@@ -408,11 +398,11 @@ _event_thread_f(void __attribute__ ((unused)) *arg)
 
 		if (ev->ev_type == EVENT_CONFIG) {
 			/*
-			log_printf(LOG_NOTICE, "Config Event: %d -> %d\n",
+			logt_print(LOG_NOTICE, "Config Event: %d -> %d\n",
 			       ev->ev.config.cfg_oldversion,
 			       ev->ev.config.cfg_version);
 			 */
-			init_resource_groups(1);
+			init_resource_groups(1, 0);
 			free(ev);
 			continue;
 		}
@@ -432,7 +422,7 @@ _event_thread_f(void __attribute__ ((unused)) *arg)
 
 		if (ev->ev_type == EVENT_RG) {
 			/*
-			log_printf(LOG_NOTICE, "RG Event: %s %s %d\n",
+			logt_print(LOG_NOTICE, "RG Event: %s %s %d\n",
 			       ev->ev.group.rg_name,
 			       rg_state_str(ev->ev.group.rg_state),
 			       ev->ev.group.rg_owner);
@@ -442,33 +432,13 @@ _event_thread_f(void __attribute__ ((unused)) *arg)
 				    ev->ev.group.rg_owner);
 		} else if (ev->ev_type == EVENT_NODE) {
 			/*
-			log_printf(LOG_NOTICE, "Node Event: %s %d %s %s\n",
+			logt_print(LOG_NOTICE, "Node Event: %s %d %s %s\n",
 			       ev->ev.node.ne_local?"Local":"Remote",
 			       ev->ev.node.ne_nodeid,
 			       ev->ev.node.ne_state?"UP":"DOWN",
 			       ev->ev.node.ne_clean?"Clean":"Dirty")
 			 */
 
-			if (ev->ev.node.ne_state == 0 &&
-			    !ev->ev.node.ne_clean &&
-			    node_has_fencing(ev->ev.node.ne_nodeid)) {
-				notice = 0;
-				while (!node_fenced(ev->ev.node.ne_nodeid)) {
-					if (!notice) {
-						notice = 1;
-						log_printf(LOG_INFO, "Waiting for "
-						       "node #%d to be fenced\n",
-						       ev->ev.node.ne_nodeid);
-					}
-					sleep(2);
-				}
-
-				if (notice)
-					log_printf(LOG_INFO, "Node #%d fenced; "
-					       "continuing\n",
-					       ev->ev.node.ne_nodeid);
-			}
-
 			node_event(ev->ev.node.ne_local,
 				   ev->ev.node.ne_nodeid,
 				   ev->ev.node.ne_state,
@@ -479,7 +449,7 @@ _event_thread_f(void __attribute__ ((unused)) *arg)
 	}
 
 	if (!central_events || _master) {
-		log_printf(LOG_DEBUG, "%d events processed\n", count);
+		logt_print(LOG_DEBUG, "%d events processed\n", count);
 	}
 	/* Mutex held */
 	event_thread = 0;
@@ -560,13 +530,11 @@ node_event_q(int local, int nodeID, int state, int clean)
 
 
 void
-config_event_q(int old_version, int new_version)
+config_event_q(void)
 {
 	event_t *ev = new_event();
 
 	ev->ev_type = EVENT_CONFIG;
-	ev->ev.config.cfg_version = new_version;
-	ev->ev.config.cfg_oldversion = old_version;
 	insert_event(ev);
 }
 
